/**************************************************************************
 *                                                                        *
 *   Copyright (c) Digital Equipment  Corporation,  1991.   All  Rights   *
 *   Reserved.  Unpublished rights reserved under the copyright laws of   *
 *   the United States.                                                   *
 *                                                                        *
 *   The software  contained  on  this  media  is  proprietary  to  and   *
 *   embodies   the   confidential   technology  of  Digital  Equipment   *
 *   Corporation.  Possession, use, duplication or dissemination of the   *
 *   software  and media is authorized only pursuant to a valid written   *
 *   license from Digital Equipment Corporation.                          *
 *                                                                        *
 *   RESTRICTED RIGHTS LEGEND Use, duplication, or  disclosure  by  the   *
 *   U.S.   Government  is  subject  to  restrictions  as  set forth in   *
 *   Subparagraph  (c)(1)(ii)  of  DFARS  252.227-7013,   or   in   FAR   *
 *   52.227-19, as applicable.                                            *
 *                                                                        *
 **************************************************************************/
#ifdef SCCS
static char sccsid[]="@(#)read_image.c	5.3 Stardent 91/03/03";
#endif
/*
			Copyright (c) 1989 by
			Stardent Computer Inc.
			All Rights Reserved
	
	This software comprises unpublished confidential information of
	Stardent Computer Inc. and may not be used, copied or made
	available to anyone, except in accordance with the license
	under which it is furnished.
	
	This file is under sccs control at Stardent in:
	/sccs3.p/avs/examples/s.read_image.c
	
*/
/******************************************************************************
 * read_image - read an image from a file:
 *         width  (4 byte int, most significant byte first)
 *         height (4 byte int, most significant byte first)
 *         data   (width * height * 4_byte_pixels[alpha red green blue])  
 *
 * NOTE: The format of this file is unnecessarily restrictive due to the 
 *       simplicity of the header.  
 *
 ******************************************************************************/

#include <stdio.h>
#include <avs/avs.h>
#include <avs/field.h>
#include <avs/flow.h>

/* Used for AVSmessage() */
static char file_version[] = "@(#)read_image.c	5.3 Stellar 91/03/03";

/******************************************************************************/
AVSinit_modules()
{
    int read_image();

    AVSmodule_from_desc(read_image);
}

/******************************************************************************/
 static
read_image()
{
    int param;
    int read_image_compute();
    
    /* Set the module name */
    AVSset_module_name("read image c", MODULE_DATA);

    /* Create the output port */
    AVScreate_output_port("Field Output", "field 2D 4-vector byte");
    
    /* Add parameter */
    param = AVSadd_parameter("Read Image Browser", "string", 0, 0, ".x");
    AVSconnect_widget(param, "browser");

    /* Set the compute procedure */
    AVSset_compute_proc(read_image_compute);
}

/******************************************************************************/	
 static
read_image_compute(data, filename)
    AVSfield_char **data;
    char *filename;
{
    AVSfield_char *input;
    FILE *pf;
    int dims[2];
    float  min_extent[2], max_extent[2];
    unsigned int cx, cy;
    unsigned char header[8];

    /* No data file name */
    if (!filename)
      return(1);

    /* Attempt to open file */
    if (!(pf = fopen(filename, "r"))) {
	AVSmessage(file_version, AVS_Warning, AVSmodule, "read_image_compute",
		   "Ok", "Can't open data file %s", filename);
	return(0);
    }
    
    /* Free old memory */
    if (*data) 
      AVSfield_free(*data);
    
    /* Read dimensions */

    if (fread(header, sizeof(header), 1, pf) != 1) {
	AVSmessage(file_version, AVS_Warning, AVSmodule, "read_image_compute",
		   "Ok", "Error reading header from file %s", filename);
	fclose(pf);
	return(0);
    }

    cx = (header[2] << 8) + header[3];
    cy = (header[6] << 8) + header[7];
   
    /* Some old image files on little endian machines might have the */
    /* bytes reversed. */

    if (cx == 0 && cy == 0) {
	cx = (header[1] << 8) + header[0];
	cy = (header[5] << 8) + header[4];
    }

    /* Set dimensions */
    dims[0] = cx;
    dims[1] = cy;
    
    /* Allocate space for image data */
    input = (AVSfield_char *)AVSdata_alloc("field 2D 4-vector byte", dims);
    
    /* Read data, 4 bytes per pixel. */
    if (fread(input->data, 4, (int)(cx*cy), pf) != (int) (cx * cy)) {
	AVSmessage(file_version, AVS_Warning, AVSmodule, "read_image_compute",
		   "Ok", "Error reading file %s", filename);
	fclose(pf);
	AVSfield_free(input);
	return(0);
    }
    
    /* Close the file */
    fclose(pf);
    
    /* Set the output pointer */
    *data = input;
    
    /* set the labels for each component */
    AVSfield_set_labels (input, "alpha;red;green;blue", ";");

    /* set the extent for the image */
    min_extent[0] = 0.0;               min_extent[1] = 0.0;
    max_extent[0] = (float) cx - 1.0;  max_extent[1] = (float) cy - 1.0;
    AVSfield_set_extent (input, min_extent, max_extent);

    /* Indicate success */
    return(1);
}
